#include <stdio.h>
#include <string.h>
#include "vars.h"

/*************************************
 * escape and print don't redirected */

int pass(void)
{
 #ifdef DEBUG_PASS
 fprintf(stderr,"Pass client request\n");
 #endif

 printf("%s %d.%d.%d.%d/%s %s %s\n",input_url,(unsigned char)input_ip[0],(unsigned char)input_ip[1],(unsigned char)input_ip[2],(unsigned char)input_ip[3],input_host,input_ident,input_method);
 fflush(stdout);

 return(0);
}
